@@ -8,13 +8,13 @@ test_elementtree
 """
 
 
-import unittest, copy, sys
+import unittest, copy, sys, operator
 
 from common_imports import etree, StringIO, HelperTestCase, fileInTestDir
 from common_imports import SillyFileLike, canonicalize, doctest
 
 print
-print "TESTED VERSION:"
+print "TESTED VERSION:", etree.__version__
 print "    Python:           ", sys.version_info
 print "    lxml.etree:       ", etree.LXML_VERSION
 print "    libxml used:      ", etree.LIBXML_VERSION
@@ -24,7 +24,7 @@ print "    libxslt compiled: ", etree.LIBXSLT_COMPILED_VERSION
 print
 
 try:
-    sorted(())
+    sorted
 except NameError:
     # Python 2.3
     def sorted(seq):
@@ -48,7 +48,13 @@ class ETreeOnlyTestCase(HelperTestCase):
             str(etree.LXML_VERSION[0])))
 
     def test_c_api(self):
-        self.assert_(hasattr(self.etree, '_import_c_api'))
+        if hasattr(self.etree, '__pyx_capi__'):
+            # newer Pyrex compatible C-API
+            self.assert_(isinstance(self.etree.__pyx_capi__, dict))
+            self.assert_(len(self.etree.__pyx_capi__) > 0)
+        else:
+            # older C-API mechanism
+            self.assert_(hasattr(self.etree, '_import_c_api'))
 
     def test_element_names(self):
         Element = self.etree.Element
@@ -74,15 +80,101 @@ class ETreeOnlyTestCase(HelperTestCase):
         el = Element('name')
         self.assertRaises(ValueError, setattr, el, 'tag', 'p:name')
 
-    def test_attribute_set(self):
-        # ElementTree accepts arbitrary attribute values
-        # lxml.etree allows only strings
+    def test_element_name_quote(self):
         Element = self.etree.Element
+        self.assertRaises(ValueError, Element, "p'name")
+        self.assertRaises(ValueError, Element, 'p"name')
 
+        self.assertRaises(ValueError, Element, "{test}p'name")
+        self.assertRaises(ValueError, Element, '{test}p"name')
+
+        el = Element('name')
+        self.assertRaises(ValueError, setattr, el, 'tag', "p'name")
+        self.assertRaises(ValueError, setattr, el, 'tag', 'p"name')
+
+    def test_element_name_space(self):
+        Element = self.etree.Element
+        self.assertRaises(ValueError, Element, ' name ')
+        self.assertRaises(ValueError, Element, 'na me')
+        self.assertRaises(ValueError, Element, '{test} name')
+
+        el = Element('name')
+        self.assertRaises(ValueError, setattr, el, 'tag', ' name ')
+
+    def test_subelement_name_empty(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        el = Element('name')
+        self.assertRaises(ValueError, SubElement, el, '{}')
+        self.assertRaises(ValueError, SubElement, el, '{test}')
+
+    def test_subelement_name_colon(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        el = Element('name')
+        self.assertRaises(ValueError, SubElement, el, 'p:name')
+        self.assertRaises(ValueError, SubElement, el, '{test}p:name')
+
+    def test_subelement_name_quote(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        el = Element('name')
+        self.assertRaises(ValueError, SubElement, el, "p'name")
+        self.assertRaises(ValueError, SubElement, el, "{test}p'name")
+
+        self.assertRaises(ValueError, SubElement, el, 'p"name')
+        self.assertRaises(ValueError, SubElement, el, '{test}p"name')
+
+    def test_subelement_name_space(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        el = Element('name')
+        self.assertRaises(ValueError, SubElement, el, ' name ')
+        self.assertRaises(ValueError, SubElement, el, 'na me')
+        self.assertRaises(ValueError, SubElement, el, '{test} name')
+
+    def test_qname_empty(self):
+        QName = self.etree.QName
+        self.assertRaises(ValueError, QName, '')
+        self.assertRaises(ValueError, QName, 'test', '')
+
+    def test_qname_colon(self):
+        QName = self.etree.QName
+        self.assertRaises(ValueError, QName, 'p:name')
+        self.assertRaises(ValueError, QName, 'test', 'p:name')
+
+    def test_qname_space(self):
+        QName = self.etree.QName
+        self.assertRaises(ValueError, QName, ' name ')
+        self.assertRaises(ValueError, QName, 'na me')
+        self.assertRaises(ValueError, QName, 'test', ' name')
+
+    def test_qname_text_resolve(self):
+        # ET doesn't resove QNames as text values
+        etree = self.etree
+        qname = etree.QName('http://myns', 'a')
+        a = etree.Element(qname, nsmap={'p' : 'http://myns'})
+        a.text = qname
+
+        self.assertEquals("p:a", a.text)
+
+    def test_attribute_set(self):
+        Element = self.etree.Element
         root = Element("root")
         root.set("attr", "TEST")
         self.assertEquals("TEST", root.get("attr"))
+
+    def test_attribute_set_invalid(self):
+        # ElementTree accepts arbitrary attribute values
+        # lxml.etree allows only strings
+        Element = self.etree.Element
+        root = Element("root")
         self.assertRaises(TypeError, root.set, "newattr", 5)
+        self.assertRaises(TypeError, root.set, "newattr", None)
 
     def test_attrib_pop(self):
         ElementTree = self.etree.ElementTree
@@ -176,16 +268,16 @@ class ETreeOnlyTestCase(HelperTestCase):
         f.close()
 
     def test_parse_remove_comments(self):
-        parse = self.etree.parse
+        fromstring = self.etree.fromstring
         tostring = self.etree.tostring
         XMLParser = self.etree.XMLParser
 
-        f = StringIO('<a><!--A--><b><!-- B --><c/></b><!--C--></a>')
+        xml = '<a><!--A--><b><!-- B --><c/></b><!--C--></a>'
         parser = XMLParser(remove_comments=True)
-        tree = parse(f, parser)
+        root = fromstring(xml, parser)
         self.assertEquals(
             '<a><b><c/></b></a>',
-            tostring(tree))
+            tostring(root))
 
     def test_parse_remove_pis(self):
         parse = self.etree.parse
@@ -200,7 +292,6 @@ class ETreeOnlyTestCase(HelperTestCase):
             xml,
             tostring(tree))
 
-        f = StringIO(xml)
         parser = XMLParser(remove_pis=True)
         tree = parse(f, parser)
         self.assertEquals(
@@ -229,6 +320,10 @@ class ETreeOnlyTestCase(HelperTestCase):
                        if 'PARSER'   in log.domain_name])
         self.assert_([ log for log in logs
                        if 'TAG_NAME_MISMATCH' in log.type_name ])
+        self.assert_([ log for log in logs
+                       if 1 == log.line ])
+        self.assert_([ log for log in logs
+                       if 15 == log.column ])
 
     def test_parse_error_from_file(self):
         parse = self.etree.parse
@@ -301,6 +396,26 @@ class ETreeOnlyTestCase(HelperTestCase):
             8,
             len(events))
 
+    def test_iterparse_encoding_8bit_override(self):
+        text = u'Søk på nettet'
+        wrong_declaration = "<?xml version='1.0' encoding='UTF-8'?>"
+        xml_latin1 = (u'%s<a>%s</a>' % (wrong_declaration, text)
+                      ).encode('iso-8859-1')
+
+        self.assertRaises(self.etree.ParseError,
+                          list, self.etree.iterparse(StringIO(xml_latin1)))
+
+        iterator = self.etree.iterparse(StringIO(xml_latin1),
+                                        encoding="iso-8859-1")
+        self.assertEquals(1, len(list(iterator)))
+
+        a = iterator.root
+        self.assertEquals(a.text, text)
+
+    def test_parser_encoding_unknown(self):
+        self.assertRaises(
+            LookupError, self.etree.XMLParser, encoding="hopefully unknown")
+
     def test_iterwalk_tag(self):
         iterwalk = self.etree.iterwalk
         root = self.etree.XML('<a><b><d/></b><c/></a>')
@@ -408,7 +523,8 @@ class ETreeOnlyTestCase(HelperTestCase):
             def resolve(self, url, id, context):
                 assertEqual(url, test_url)
                 return self.resolve_string(
-                    u'<!ENTITY myentity "%s">' % url, context)
+                    u'''<!ENTITY myentity "%s">
+                        <!ELEMENT doc ANY>''' % url, context)
 
         parser.resolvers.add(MyResolver())
 
@@ -419,7 +535,7 @@ class ETreeOnlyTestCase(HelperTestCase):
 
     def test_resolve_empty(self):
         parse = self.etree.parse
-        parser = self.etree.XMLParser(dtd_validation=True)
+        parser = self.etree.XMLParser(load_dtd=True)
         assertEqual = self.assertEqual
         test_url = u"__nosuch.dtd"
 
@@ -435,12 +551,9 @@ class ETreeOnlyTestCase(HelperTestCase):
         parser.resolvers.add(MyResolver())
 
         xml = u'<!DOCTYPE doc SYSTEM "%s"><doc>&myentity;</doc>' % test_url
-        tree = parse(StringIO(xml), parser)
+        self.assertRaises(etree.XMLSyntaxError, parse, StringIO(xml), parser)
         self.assert_(check.resolved)
 
-        root = tree.getroot()
-        self.assertEquals(root.text, None)
-
     def test_resolve_error(self):
         parse = self.etree.parse
         parser = self.etree.XMLParser(dtd_validation=True)
@@ -458,6 +571,55 @@ class ETreeOnlyTestCase(HelperTestCase):
         xml = u'<!DOCTYPE doc SYSTEM "test"><doc>&myentity;</doc>'
         self.assertRaises(_LocalException, parse, StringIO(xml), parser)
 
+    if etree.LIBXML_VERSION > (2,6,20):
+        def test_entity_parse(self):
+            parse = self.etree.parse
+            tostring = self.etree.tostring
+            parser = self.etree.XMLParser(resolve_entities=False)
+            Entity = self.etree.Entity
+
+            xml = '<!DOCTYPE doc SYSTEM "test"><doc>&myentity;</doc>'
+            tree = parse(StringIO(xml), parser)
+            root = tree.getroot()
+            self.assertEquals(root[0].tag, Entity)
+            self.assertEquals(root[0].text, "&myentity;")
+            self.assertEquals(root[0].tail, None)
+            self.assertEquals(root[0].name, "myentity")
+
+            self.assertEquals('<doc>&myentity;</doc>',
+                              tostring(root))
+
+    def test_entity_append(self):
+        Entity = self.etree.Entity
+        Element = self.etree.Element
+        tostring = self.etree.tostring
+
+        root = Element("root")
+        root.append( Entity("test") )
+
+        self.assertEquals(root[0].tag, Entity)
+        self.assertEquals(root[0].text, "&test;")
+        self.assertEquals(root[0].tail, None)
+        self.assertEquals(root[0].name, "test")
+
+        self.assertEquals('<root>&test;</root>',
+                          tostring(root))
+
+    def test_entity_values(self):
+        Entity = self.etree.Entity
+        self.assertEquals(Entity("test").text, '&test;')
+        self.assertEquals(Entity("#17683").text, '&#17683;')
+        self.assertEquals(Entity("#x1768").text, '&#x1768;')
+        self.assertEquals(Entity("#x98AF").text, '&#x98AF;')
+
+    def test_entity_error(self):
+        Entity = self.etree.Entity
+        self.assertRaises(ValueError, Entity, 'a b c')
+        self.assertRaises(ValueError, Entity, 'a,b')
+        self.assertRaises(AssertionError, Entity, 'a\0b')
+        self.assertRaises(ValueError, Entity, '#abc')
+        self.assertRaises(ValueError, Entity, '#xxyz')
+
     # TypeError in etree, AssertionError in ElementTree;
     def test_setitem_assert(self):
         Element = self.etree.Element
@@ -1177,13 +1339,13 @@ class ETreeOnlyTestCase(HelperTestCase):
 
     def test_namespaces_reuse_after_move(self):
         ns_href = "http://a.b.c"
-        one = self.etree.parse(
-            StringIO('<foo><bar xmlns:ns="%s"><ns:baz/></bar></foo>' % ns_href))
-        baz = one.getroot()[0][0]
+        one = self.etree.fromstring(
+            '<foo><bar xmlns:ns="%s"><ns:baz/></bar></foo>' % ns_href)
+        baz = one[0][0]
 
-        two = self.etree.parse(
-            StringIO('<root xmlns:ns="%s"/>' % ns_href))
-        two.getroot().append(baz)
+        two = self.etree.fromstring(
+            '<root xmlns:ns="%s"/>' % ns_href)
+        two.append(baz)
         del one # make sure the source document is deallocated
 
         self.assertEquals('{%s}baz' % ns_href, baz.tag)
@@ -1191,8 +1353,7 @@ class ETreeOnlyTestCase(HelperTestCase):
             '<root xmlns:ns="%s"><ns:baz/></root>' % ns_href,
             self.etree.tostring(two))
 
-    def _test_namespaces_after_serialize(self):
-        # FIXME: this currently fails - fix serializer.pxi!
+    def test_namespaces_after_serialize(self):
         parse = self.etree.parse
         tostring = self.etree.tostring
 
@@ -1201,9 +1362,7 @@ class ETreeOnlyTestCase(HelperTestCase):
             StringIO('<foo><bar xmlns:ns="%s"><ns:baz/></bar></foo>' % ns_href))
         baz = one.getroot()[0][0]
 
-        print tostring(baz)
         parsed = parse(StringIO( tostring(baz) )).getroot()
-
         self.assertEquals('{%s}baz' % ns_href, parsed.tag)
 
     def test_element_nsmap(self):
@@ -1263,6 +1422,31 @@ class ETreeOnlyTestCase(HelperTestCase):
             [d, f],
             list(a.getiterator('{b}*')))
 
+    def test_getiterator_filter_entities(self):
+        Element = self.etree.Element
+        Entity = self.etree.Entity
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        entity_b = Entity("TEST-b")
+        b.append(entity_b)
+
+        self.assertEquals(
+            [entity_b],
+            list(a.getiterator(Entity)))
+
+        entity_a = Entity("TEST-a")
+        a.append(entity_a)
+
+        self.assertEquals(
+            [entity_b, entity_a],
+            list(a.getiterator(Entity)))
+
+        self.assertEquals(
+            [entity_b],
+            list(b.getiterator(Entity)))
+
     def test_getiterator_filter_element(self):
         Element = self.etree.Element
         Comment = self.etree.Comment
@@ -1296,6 +1480,24 @@ class ETreeOnlyTestCase(HelperTestCase):
             [a, b, c],
             list(a.getiterator('*')))
 
+    def test_itertext(self):
+        # ET 1.3+
+        XML = self.etree.XML
+        root = XML("<root>RTEXT<a></a>ATAIL<b/><c>CTEXT</c>CTAIL</root>")
+
+        text = list(root.itertext())
+        self.assertEquals(["RTEXT", "ATAIL", "CTEXT", "CTAIL"],
+                          text)
+
+    def test_itertext_child(self):
+        # ET 1.3+
+        XML = self.etree.XML
+        root = XML("<root>RTEXT<a></a>ATAIL<b/><c>CTEXT</c>CTAIL</root>")
+
+        text = list(root[2].itertext())
+        self.assertEquals(["CTEXT"],
+                          text)
+
     def test_findall_ns(self):
         XML = self.etree.XML
         root = XML('<a xmlns:x="X" xmlns:y="Y"><x:b><c/></x:b><b/><c><x:b/><b/></c><b/></a>')
@@ -1387,6 +1589,103 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertEquals(
             child1, e[1])
 
+    def test_setslice_all_empty_reversed(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+
+        e = Element('e')
+        f = Element('f')
+        g = Element('g')
+
+        s = [e, f, g]
+        a[::-1] = s
+        self.assertEquals(
+            [g, f, e],
+            list(a))
+
+    def test_setslice_step(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+        d = SubElement(a, 'd')
+        e = SubElement(a, 'e')
+
+        x = Element('x')
+        y = Element('y')
+
+        a[1::2] = [x, y]
+        self.assertEquals(
+            [b, x, d, y],
+            list(a))
+
+    def test_setslice_step_negative(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+        d = SubElement(a, 'd')
+        e = SubElement(a, 'e')
+
+        x = Element('x')
+        y = Element('y')
+
+        a[1::-1] = [x, y]
+        self.assertEquals(
+            [y, x, d, e],
+            list(a))
+
+    def test_setslice_step_negative2(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+        d = SubElement(a, 'd')
+        e = SubElement(a, 'e')
+
+        x = Element('x')
+        y = Element('y')
+
+        a[::-2] = [x, y]
+        self.assertEquals(
+            [b, y, d, x],
+            list(a))
+
+    def test_setslice_step_overrun(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        try:
+            slice
+        except NameError:
+            print "slice() not found"
+            return
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+        d = SubElement(a, 'd')
+        e = SubElement(a, 'e')
+
+        x = Element('x')
+        y = Element('y')
+        z = Element('z')
+
+        self.assertRaises(
+            ValueError,
+            operator.setitem, a, slice(1,None,2), [x, y, z])
+
+        self.assertEquals(
+            [b, c, d, e],
+            list(a))
+
     def test_extend(self):
         etree = self.etree
         root = etree.Element('foo')
@@ -1435,6 +1734,25 @@ class ETreeOnlyTestCase(HelperTestCase):
             [1, 2, 3],
             [ el.sourceline for el in tree.getiterator() ])
 
+    def test_sourceline_iterparse_end(self):
+        iterparse = self.etree.iterparse
+        lines = [ el.sourceline for (event, el) in 
+                  iterparse(fileInTestDir('include/test_xinclude.xml')) ]
+
+        self.assertEquals(
+            [2, 3, 1],
+            lines)
+
+    def test_sourceline_iterparse_start(self):
+        iterparse = self.etree.iterparse
+        lines = [ el.sourceline for (event, el) in 
+                  iterparse(fileInTestDir('include/test_xinclude.xml'),
+                            events=("start",)) ]
+
+        self.assertEquals(
+            [1, 2, 3],
+            lines)
+
     def test_sourceline_element(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -1503,6 +1821,20 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertEquals(docinfo.root_name,   'html')
         self.assertEquals(docinfo.doctype, '')
 
+    def test_dtd_io(self):
+        # check that DTDs that go in also go back out
+        xml = '''\
+        <!DOCTYPE test SYSTEM "test.dtd" [
+          <!ENTITY entity "tasty">
+          <!ELEMENT test (a)>
+          <!ELEMENT a (#PCDATA)>
+        ]>
+        <test><a>test-test</a></test>\
+        '''
+        tree = self.etree.parse(StringIO(xml))
+        self.assertEqual(self.etree.tostring(tree).replace(" ", ""),
+                         xml.replace(" ", ""))
+
     def test_byte_zero(self):
         Element = self.etree.Element
 
@@ -1557,7 +1889,7 @@ class ETreeOnlyTestCase(HelperTestCase):
         b = SubElement(a, 'b')
         c = SubElement(a, 'c')
 
-        result = unicode(tostring(a, 'UTF-16'), 'UTF-16')
+        result = unicode(tostring(a, encoding='UTF-16'), 'UTF-16')
         self.assertEquals('<a><b></b><c></c></a>',
                           canonicalize(result))
 
@@ -1582,7 +1914,46 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertEquals(result, "<a><b/><c/></a>")
 
         result = tostring(a, pretty_print=True)
-        self.assertEquals(result, "<a>\n  <b/>\n  <c/>\n</a>")
+        self.assertEquals(result, "<a>\n  <b/>\n  <c/>\n</a>\n")
+
+    def test_tostring_with_tail(self):
+        tostring = self.etree.tostring
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        a.tail = "aTAIL"
+        b = SubElement(a, 'b')
+        b.tail = "bTAIL"
+        c = SubElement(a, 'c')
+
+        result = tostring(a)
+        self.assertEquals(result, "<a><b/>bTAIL<c/></a>aTAIL")
+
+        result = tostring(a, with_tail=False)
+        self.assertEquals(result, "<a><b/>bTAIL<c/></a>")
+
+        result = tostring(a, with_tail=True)
+        self.assertEquals(result, "<a><b/>bTAIL<c/></a>aTAIL")
+
+    def test_tostring_method_text_encoding(self):
+        tostring = self.etree.tostring
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        
+        a = Element('a')
+        a.text = "A"
+        a.tail = "tail"
+        b = SubElement(a, 'b')
+        b.text = "B"
+        b.tail = u"Søk på nettet"
+        c = SubElement(a, 'c')
+        c.text = "C"
+
+        result = tostring(a, method="text", encoding="UTF-16")
+
+        self.assertEquals(u'ABSøk på nettetCtail'.encode("UTF-16"),
+                          result)
 
     def test_tounicode(self):
         tounicode = self.etree.tounicode
@@ -1648,7 +2019,76 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertEquals(result, "<a><b/><c/></a>")
 
         result = tounicode(a, pretty_print=True)
-        self.assertEquals(result, "<a>\n  <b/>\n  <c/>\n</a>")
+        self.assertEquals(result, "<a>\n  <b/>\n  <c/>\n</a>\n")
+
+    def test_tostring_unicode(self):
+        tostring = self.etree.tostring
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+        
+        self.assert_(isinstance(tostring(a, encoding=unicode), unicode))
+        self.assertEquals('<a><b></b><c></c></a>',
+                          canonicalize(tostring(a, encoding=unicode)))
+
+    def test_tostring_unicode_element(self):
+        tostring = self.etree.tostring
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+        d = SubElement(c, 'd')
+        self.assert_(isinstance(tostring(b, encoding=unicode), unicode))
+        self.assert_(isinstance(tostring(c, encoding=unicode), unicode))
+        self.assertEquals('<b></b>',
+                          canonicalize(tostring(b, encoding=unicode)))
+        self.assertEquals('<c><d></d></c>',
+                          canonicalize(tostring(c, encoding=unicode)))
+
+    def test_tostring_unicode_none(self):
+        tostring = self.etree.tostring
+        self.assertRaises(TypeError, self.etree.tostring,
+                          None, encoding=unicode)
+
+    def test_tostring_unicode_element_tail(self):
+        tostring = self.etree.tostring
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+        d = SubElement(c, 'd')
+        b.tail = 'Foo'
+
+        self.assert_(isinstance(tostring(b, encoding=unicode), unicode))
+        self.assert_(tostring(b, encoding=unicode) == '<b/>Foo' or
+                     tostring(b, encoding=unicode) == '<b />Foo')
+
+    def test_tostring_unicode_pretty(self):
+        tostring = self.etree.tostring
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+
+        result = tostring(a, encoding=unicode)
+        self.assertEquals(result, "<a><b/><c/></a>")
+
+        result = tostring(a, encoding=unicode, pretty_print=False)
+        self.assertEquals(result, "<a><b/><c/></a>")
+
+        result = tostring(a, encoding=unicode, pretty_print=True)
+        self.assertEquals(result, "<a>\n  <b/>\n  <c/>\n</a>\n")
+
+    # helper methods
 
     def _writeElement(self, element, encoding='us-ascii'):
         """Write out element for comparison.
@@ -1656,7 +2096,7 @@ class ETreeOnlyTestCase(HelperTestCase):
         ElementTree = self.etree.ElementTree
         f = StringIO()
         tree = ElementTree(element=element)
-        tree.write(f, encoding)
+        tree.write(f, encoding=encoding)
         data = f.getvalue()
         return canonicalize(data)
 
@@ -1679,6 +2119,9 @@ class XIncludeTestCase(HelperTestCase):
 
     def test_xinclude(self):
         tree = etree.parse(fileInTestDir('include/test_xinclude.xml'))
+        self.assertNotEquals(
+            'a',
+            tree.getroot()[1].tag)
         # process xincludes
         self.include( tree )
         # check whether we find it replaced with included data
@@ -1723,4 +2166,4 @@ def test_suite():
     return suite
 
 if __name__ == '__main__':
-    unittest.main()
+    print 'to test use test.py %s' % __file__
